<?php

namespace App\Http\Controllers\Client;

use App\Http\Controllers\Controller;
use App\Http\Requests\UpdateProfileRequest;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;

class ProfileController extends Controller
{
    /**
     *  @OA\Get(
     *    tags={"Clients:Authentication"},
     *    path="/auth/profile",
     *    summary="Get user profile",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="user",
     *          allOf={
     *            @OA\Schema(ref="#/components/schemas/User"),
     *            @OA\Schema(
     *              @OA\Property(
     *                property="server",
     *                allOf={
     *                  @OA\Schema(ref="#/components/schemas/Server"),
     *                  @OA\Schema(nullable=true)
     *                }
     *              ),
     *              @OA\Property(type="string", property="email", format="email"),
     *            )
     *          }
     *        ),
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function show()
    {
        return response()->json([
            'user' => Auth::user()->load('server')->makeVisible('email'),
        ]);
    }

    /**
     *  @OA\Put(
     *    tags={"Clients:Authentication"},
     *    path="/auth/profile",
     *    summary="Update user profile",
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/UpdateProfile"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK"
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function update(UpdateProfileRequest $request)
    {
        $data = $request->validated();
        if (array_key_exists('password', $data))
            $data['password'] = Hash::make($data['password']);

        Auth::user()->update($data);
        return response()->noContent();
    }

    /**
     *  @OA\Delete(
     *    tags={"Clients:Authentication"},
     *    path="/auth/profile",
     *    summary="Delete user profile",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK"
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function destroy()
    {
        $user = Auth::user();
        Auth::logout();
        $user->delete();
        return response()->noContent();
    }
}
