<?php

namespace App\Http\Controllers\Server;

use App\Http\Controllers\Controller;
use App\Http\Requests\ServerHeartbeatRequest;
use App\Http\Requests\ServerLoginRequest;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class AuthController extends Controller
{
    /**
     *  @OA\Post(
     *    tags={"Servers:Authentication"},
     *    path="/auth/server/login",
     *    summary="Login to the API",
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/ServerLogin"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="token",
     *          type="string",
     *          description="Token to use as Bearer token",
     *        ),
     *        @OA\Property(
     *          property="server",
     *          type="object",
     *          ref="#/components/schemas/Server"
     *        ),
     *      ),
     *    ),
     *  )
     */
    public function login(ServerLoginRequest $request)
    {
        $token = Auth::guard('server')->attempt($request->validated());

        if (!$token) {
            return response()->json([
                'message' => 'Unauthorized',
            ], 401);
        }

        $server = Auth::guard('server')->user();
        return response()->json([
            'server' => $server,
            'token' => $token,
        ]);
    }

    /**
     *  @OA\Post(
     *    tags={"Servers:Authentication"},
     *    path="/auth/server/logout",
     *    summary="Logout of the API",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK",
     *    ),
     *    security={{"server-jwt":{}}},
     *  )
     */
    public function logout()
    {
        Auth::guard('server')->logout();
        return response()->noContent();
    }

    /**
     *  @OA\Post(
     *    tags={"Servers:Authentication"},
     *    path="/auth/server/refresh",
     *    summary="Refresh an API token",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="token",
     *          type="string",
     *          description="Token to use as Bearer token",
     *        ),
     *        @OA\Property(
     *          property="server",
     *          type="object",
     *          ref="#/components/schemas/Server"
     *        ),
     *      ),
     *    ),
     *    security={{"server-jwt":{}}},
     *  )
     */
    public function refresh()
    {
        return response()->json([
            'server' => Auth::guard('server')->user(),
            'token' => Auth::guard('server')->refresh(),
        ]);
    }

    /**
     *  @OA\Get(
     *    tags={"Servers:Authentication"},
     *    path="/auth/server/profile",
     *    summary="Get server profile",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="server",
     *          ref="#/components/schemas/Server",
     *        ),
     *      ),
     *    ),
     *    security={{"server-jwt":{}}},
     *  )
     */
    public function profile()
    {
        return response()->json([
            'server' => Auth::user(),
        ]);
    }

    /**
     *  @OA\Post(
     *    tags={"Servers:Authentication"},
     *    path="/auth/server/heartbeat",
     *    summary="Singla to the API that the server is online",
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/ServerHeartbeat"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK",
     *    ),
     *  )
     */
    public function heartbeat(ServerHeartbeatRequest $request)
    {
        $server = Auth::guard('server')->user();
        $server->heartbeat($request->input("ip"), $request->input("port"), $request->input("pubkey"));
        return response()->noContent();
    }

    /**
     *  @OA\Post(
     *    tags={"Servers:Authentication"},
     *    path="/auth/server/stop",
     *    summary="Singla to the API that the server has stopped",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="403",
     *      description="Server is offline",
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK",
     *    ),
     *  )
     */
    public function stop(Request $request)
    {
        $server = Auth::guard('server')->user();

        if (!$server->online) {
            return response()->json([
                'message' => 'Server is offline',
            ], 403);
        }

        $server->stop();
        return response()->noContent();
    }
}
