<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Laravel\Sanctum\HasApiTokens;
use PHPOpenSourceSaver\JWTAuth\Contracts\JWTSubject;

/**
 * @OA\Schema(
 *   title="Server",
 *   type="object",
 *   @OA\Property(type="integer", property="id"),
 *   @OA\Property(type="integer", property="owner_id"),
 *   @OA\Property(type="string", property="login"),
 *   @OA\Property(type="boolean", property="public"),
 *   @OA\Property(type="boolean", property="online"),
 *   @OA\Property(type="string", property="created_at", format="date-time"),
 *   @OA\Property(type="string", property="updated_at", format="date-time")
 * )
 */
class Server extends Authenticatable implements JWTSubject
{
    use HasApiTokens, HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'owner_id',
        'login',
        'password',
        'public',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'last_heartbeat_at',
        'ip',
        'port',
        'pubkey'
    ];

    protected $casts = [
        'public' => 'boolean',
        'online' => 'boolean',
        'last_heartbeat_at' => 'datetime'
    ];

    public function owner(): BelongsTo
    {
        return $this->belongsTo(User::class, 'owner_id');
    }

    public function users()
    {
        return $this->hasMany(User::class, 'server_id');
    }

    public function games()
    {
        return $this->hasMany(Game::class, 'server_id');
    }

    /**
     * Get the identifier that will be stored in the subject claim of the JWT.
     *
     * @return mixed
     */
    public function getJWTIdentifier()
    {
        return $this->getKey();
    }

    /**
     * Return a key value array, containing any custom claims to be added to the JWT.
     *
     * @return array
     */
    public function getJWTCustomClaims()
    {
        return [];
    }

    public function heartbeat(string $ip, int $port, string $pubkey)
    {
        $this->last_heartbeat_at = Carbon::now();
        $this->online = true;
        $this->ip = $ip;
        $this->port = $port;
        $this->pubkey = $pubkey;
        $this->save();
    }

    public function offline()
    {
        $this->last_heartbeat_at = null;
        $this->online = false;
        $this->ip = null;
        $this->port = null;
        $this->pubkey = null;
        $this->save();
    }

    public function stop()
    {
        // We fetch all the users connected to the server and mark them as disconnected.
        // NOTE: The users will have to wait at least one minute and at most three minutes
        // since the server crashes before connecting to another server.
        $this->users->each(function ($user) {
            $user->disconnect();
        });

        // We fetch all the game that were running, detach their users (thus deleting the pivot data)
        // and delete the game. This makes it as the game never happened.
        $this->games->whereIn("state", ["created", "playing"])->each(function($game) {
            $game->users()->detach();
            $game->delete();
        });

        // We mark the server as offline
        $this->offline();
    }
}