<?php

namespace Tests\Feature;

use App\Models\Game;
use App\Models\Server;
use App\Models\User;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Tests\TestCase;

class GameTest extends TestCase
{
    use RefreshDatabase;

    public function test_index()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "public" => 1,
            "owner_id" => $user->id
        ]);
        
        Game::create([
            "server_id" => $server->id
        ]);

        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/game');
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "games" => [
                '*' => [
                    "id",
                    "server_id",
                    "created_at",
                    "updated_at"
                ]
            ]
        ]);
    }

    public function test_show_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/game/999');
        $response->assertStatus(404);
    }

    public function test_show_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "public" => 1,
            "owner_id" => $user->id
        ]);
        
        $game = Game::create([
            "server_id" => $server->id
        ]);
        $game->state = 'finished';
        $game->save();
        $game->users()->attach($user);

        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/game/' . $game->id);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "game" => [
                "id",
                "server_id",
                "server" => [
                    "id",
                    "login",
                    "public",
                    "created_at",
                    "updated_at",
                    "owner_id"
                ],
            ]
        ]);
    }
    public function test_index_users_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/game/999/user');
        $response->assertStatus(404);
    }

    public function test_index_users_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "public" => 1,
            "owner_id" => $user->id
        ]);
        
        $game = Game::create([
            "server_id" => $server->id
        ]);
        $game->state = 'finished';
        $game->save();
        $game->users()->attach($user);

        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/game/' . $game->id . '/user');
        $response->assertStatus(200);
        $response->assertJsonStructure([
                "users" => [
                    "*" => [
                        "id",
                        "username",
                        "created_at",
                        "updated_at",
                        "pivot" => [
                            "game_id",
                            "user_id"
                        ]
                    ]
                ]
        ]);
    }

    public function test_show_users_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/game/999/user/9999');
        $response->assertStatus(404);
    }

    public function test_show_users_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "public" => 1,
            "owner_id" => $user->id
        ]);
        
        $game = Game::create([
            "server_id" => $server->id
        ]);
        $game->state = 'finished';
        $game->save();
        $game->users()->attach($user);

        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/game/' . $game->id . '/user/' . $user->id);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "user" => [
                "id",
                "username",
                "created_at",
                "updated_at",
                "pivot" => [
                    "game_id",
                    "user_id"
                ]
            ]
        ]);
    }
}
